(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@thi.ng/shader-ast')) :
    typeof define === 'function' && define.amd ? define(['exports', '@thi.ng/shader-ast'], factory) :
    (global = global || self, factory((global.thi = global.thi || {}, global.thi.ng = global.thi.ng || {}, global.thi.ng.shaderAstStdlib = {}), global.thi.ng.shaderAst));
}(this, function (exports, shaderAst) { 'use strict';

    const GAMMA = shaderAst.float(2.2);
    const INV_GAMMA = shaderAst.float(1 / 2.2);
    const $ = (t, x) => ({
        float: x,
        vec2: shaderAst.vec2(x),
        vec3: shaderAst.vec3(x),
        vec4: shaderAst.vec4(x, x, x, 1)
    }[t.type]);
    const toLinear = (x) => shaderAst.pow(x, $(x, GAMMA));
    const toSRGB = (x) => shaderAst.pow(x, $(x, INV_GAMMA));

    const luminanceRGB = (rgb) => shaderAst.dot(rgb, shaderAst.vec3(0.299, 0.587, 0.114));

    const clamp01 = (x) => ((x.type === "float"
        ? shaderAst.clamp(x, shaderAst.FLOAT0, shaderAst.FLOAT1)
        : x.type === "vec2"
            ? shaderAst.clamp(x, shaderAst.vec2(), shaderAst.vec2(1))
            : x.type === "vec3"
                ? shaderAst.clamp(x, shaderAst.vec3(), shaderAst.vec3(1))
                : shaderAst.clamp(x, shaderAst.vec4(), shaderAst.vec4(1))));
    const clamp11 = (x) => ((x.type === "float"
        ? shaderAst.clamp(x, shaderAst.float(-1), shaderAst.FLOAT1)
        : x.type === "vec2"
            ? shaderAst.clamp(x, shaderAst.vec2(-1), shaderAst.vec2(1))
            : x.type === "vec3"
                ? shaderAst.clamp(x, shaderAst.vec3(-1), shaderAst.vec3(1))
                : shaderAst.clamp(x, shaderAst.vec4(-1), shaderAst.vec4(1))));

    const fogExp = shaderAst.defn("float", "fogExp", [["float"], ["float"]], (dist, density) => [shaderAst.ret(shaderAst.sub(shaderAst.FLOAT1, clamp01(shaderAst.exp(shaderAst.mul(shaderAst.neg(density), dist)))))]);

    const fogExp2 = shaderAst.defn("float", "fogExp2", [["float"], ["float"]], (dist, density) => [
        shaderAst.assign(density, shaderAst.mul(density, dist)),
        shaderAst.ret(shaderAst.sub(shaderAst.FLOAT1, clamp01(shaderAst.exp2(shaderAst.mul(shaderAst.mul(density, density), -Math.LOG2E)))))
    ]);

    const fogLinear = shaderAst.defn("float", "fogLinear", [["float"], ["float"], ["float"]], (dist, start, end) => [
        shaderAst.ret(shaderAst.sub(shaderAst.FLOAT1, clamp01(shaderAst.div(shaderAst.sub(end, dist), shaderAst.sub(end, start)))))
    ]);

    const fit01 = (a, b, c) => shaderAst.mix(b, c, a);
    const fit1101 = (x) => shaderAst.add(shaderAst.mul(x, shaderAst.FLOAT05), shaderAst.FLOAT05);
    const fit0111 = (x) => shaderAst.sub(shaderAst.mul(x, shaderAst.FLOAT2), shaderAst.FLOAT1);

    const lambert = (n, ldir) => shaderAst.max(shaderAst.dot(n, ldir), shaderAst.FLOAT0);
    const halfLambert = (n, ldir) => fit1101(shaderAst.dot(n, ldir));
    const diffuseLighting = (lambertian, diffuse, light, ambient) => shaderAst.madd(shaderAst.mul(light, lambertian), diffuse, ambient);

    const trilight = shaderAst.defn("vec3", "trilight", [["vec3"], ["vec3"], ["vec3"], ["vec3"], ["vec3"]], (n, l, c1, c2, c3) => {
        let d;
        return [
            (d = shaderAst.sym(shaderAst.dot(n, l))),
            shaderAst.ret(shaderAst.add(shaderAst.add(shaderAst.mul(c1, shaderAst.max(d, shaderAst.FLOAT0)), shaderAst.mul(c2, shaderAst.sub(shaderAst.FLOAT1, shaderAst.abs(d)))), shaderAst.mul(c3, shaderAst.max(shaderAst.dot(shaderAst.neg(n), l), shaderAst.FLOAT0))))
        ];
    });

    const additive = (type, fn, oct = 4, name = "additive") => shaderAst.defn("float", name, [[type], [type], ["float"]], (pos, shift, decay) => {
        let n;
        let amp;
        return [
            (n = shaderAst.sym(shaderAst.FLOAT0)),
            (amp = shaderAst.sym(shaderAst.FLOAT05)),
            shaderAst.forLoop(shaderAst.sym("float", shaderAst.float(0)), (i) => shaderAst.lt(i, shaderAst.float(oct)), shaderAst.inc, (i) => [
                shaderAst.assign(n, shaderAst.add(n, shaderAst.mul(amp, fn(shaderAst.add(pos, shaderAst.mul(i, shift)))))),
                shaderAst.assign(amp, shaderAst.mul(amp, decay)),
                shaderAst.assign(pos, shaderAst.mul(pos, 2))
            ]),
            shaderAst.ret(n)
        ];
    });

    const sincos = (x) => shaderAst.vec2(shaderAst.sin(x), shaderAst.cos(x));
    const cossin = (x) => shaderAst.vec2(shaderAst.cos(x), shaderAst.sin(x));

    const cartesian2 = (v) => shaderAst.mul(cossin(shaderAst.$y(v)), shaderAst.vec2(shaderAst.$x(v)));
    const cartesian3 = shaderAst.defn("vec3", "cartesian3", [["vec3"]], (v) => {
        let r;
        let t;
        let p;
        return [
            (r = shaderAst.sym(shaderAst.$x(v))),
            (t = shaderAst.sym(cossin(shaderAst.$y(v)))),
            (p = shaderAst.sym(cossin(shaderAst.$z(v)))),
            shaderAst.ret(shaderAst.vec3(shaderAst.mul(shaderAst.mul(r, shaderAst.$x(t)), shaderAst.$x(p))))
        ];
    });

    const cross2 = (a, b) => crossC2(shaderAst.$x(a), shaderAst.$y(a), shaderAst.$x(b), shaderAst.$y(b));
    const crossC2 = (ax, ay, bx, by) => shaderAst.sub(shaderAst.mul(ax, by), shaderAst.mul(ay, bx));

    const distChebyshev2 = (a, b) => shaderAst.max(shaderAst.abs(shaderAst.sub(shaderAst.$x(a), shaderAst.$x(b))), shaderAst.abs(shaderAst.sub(shaderAst.$y(a), shaderAst.$y(b))));
    const distChebyshev3 = (a, b) => shaderAst.max(distChebyshev2(a, b), shaderAst.abs(shaderAst.sub(shaderAst.$z(a), shaderAst.$z(b))));
    const distChebyshev4 = (a, b) => shaderAst.max(distChebyshev3(a, b), shaderAst.abs(shaderAst.sub(shaderAst.$w(a), shaderAst.$w(b))));

    const distManhattan2 = (a, b) => shaderAst.add(shaderAst.abs(shaderAst.sub(shaderAst.$x(a), shaderAst.$x(b))), shaderAst.abs(shaderAst.sub(shaderAst.$y(a), shaderAst.$y(b))));
    const distManhattan3 = (a, b) => shaderAst.add(distManhattan2(a, b), shaderAst.abs(shaderAst.sub(shaderAst.$z(a), shaderAst.$z(b))));
    const distManhattan4 = (a, b) => shaderAst.add(distManhattan3(a, b), shaderAst.abs(shaderAst.sub(shaderAst.$w(a), shaderAst.$w(b))));

    const magSq2 = shaderAst.defn("float", "magSq2", [["vec2"]], (v) => [
        shaderAst.ret(shaderAst.dot(v, v))
    ]);
    const magSq3 = shaderAst.defn("float", "magSq3", [["vec3"]], (v) => [
        shaderAst.ret(shaderAst.dot(v, v))
    ]);
    const magSq4 = shaderAst.defn("float", "magSq4", [["vec4"]], (v) => [
        shaderAst.ret(shaderAst.dot(v, v))
    ]);

    const maxComp2 = (v) => shaderAst.max(shaderAst.$x(v), shaderAst.$y(v));
    const maxComp3 = (v) => shaderAst.max(maxComp2(v), shaderAst.$z(v));
    const maxComp4 = (v) => shaderAst.max(maxComp3(v), shaderAst.$w(v));

    const minComp2 = (v) => shaderAst.min(shaderAst.$x(v), shaderAst.$y(v));
    const minComp3 = (v) => shaderAst.min(minComp2(v), shaderAst.$z(v));
    const minComp4 = (v) => shaderAst.min(minComp3(v), shaderAst.$w(v));

    const perpendicularCCW = (v) => shaderAst.vec2(shaderAst.neg(shaderAst.$y(v)), shaderAst.$x(v));
    const perpendicularCW = (v) => shaderAst.vec2(shaderAst.$y(v), shaderAst.neg(shaderAst.$x(v)));
    const orthogonal3 = shaderAst.defn("vec3", "orthogonal3", [["vec3"]], (v) => [
        shaderAst.ret(shaderAst.ternary(shaderAst.gt(shaderAst.abs(shaderAst.$x(v)), shaderAst.abs(shaderAst.$z(v))), shaderAst.vec3(shaderAst.neg(shaderAst.$y(v)), shaderAst.$x(v), 0), shaderAst.vec3(0, shaderAst.neg(shaderAst.$z(v)), shaderAst.$y(v))))
    ]);

    const polar2 = shaderAst.defn("vec2", "polar2", [["vec2"]], (v) => [
        shaderAst.ret(shaderAst.vec2(shaderAst.length(v), shaderAst.atan(shaderAst.div(shaderAst.$y(v), shaderAst.$x(v)))))
    ]);
    const polar3 = shaderAst.defn("vec3", "polar3", [["vec3"]], (v) => {
        let r;
        return [
            (r = shaderAst.sym(shaderAst.length(v))),
            shaderAst.ret(shaderAst.vec3(r, shaderAst.asin(shaderAst.div(shaderAst.$z(v), r)), shaderAst.atan(shaderAst.div(shaderAst.$y(v), shaderAst.$x(v)))))
        ];
    });

    const lookat = shaderAst.defn("mat4", "lookat", [["vec3"], ["vec3"], ["vec3"]], (eye, target, up) => {
        let x;
        let y;
        let z;
        return [
            (z = shaderAst.sym(shaderAst.normalize(shaderAst.sub(eye, target)))),
            (x = shaderAst.sym(shaderAst.normalize(shaderAst.cross(up, z)))),
            (y = shaderAst.sym(shaderAst.normalize(shaderAst.cross(z, x)))),
            shaderAst.ret(shaderAst.mat4(shaderAst.vec4(x, shaderAst.neg(shaderAst.dot(eye, x))), shaderAst.vec4(up, shaderAst.neg(shaderAst.dot(eye, y))), shaderAst.vec4(z, shaderAst.neg(shaderAst.dot(eye, z))), shaderAst.vec4(0, 0, 0, 1)))
        ];
    });

    const transformMVP = (p, model, view, proj) => shaderAst.mul(shaderAst.mul(proj, shaderAst.mul(view, model)), shaderAst.vec4(p, shaderAst.FLOAT1));

    const surfaceNormal = (n, normalMat) => shaderAst.normalize(shaderAst.$xyz(shaderAst.mul(normalMat, shaderAst.vec4(n, 0))));

    const rotation2 = shaderAst.defn("mat2", "rotation2", [["float"]], (theta) => {
        let cs;
        return [(cs = shaderAst.sym(cossin(theta))), shaderAst.ret(shaderAst.mat2(cs, perpendicularCCW(cs)))];
    });
    const rotationX3 = shaderAst.defn("mat3", "rotationX3", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat3(1, 0, 0, 0, shaderAst.$x(cs), shaderAst.$y(cs), 0, shaderAst.neg(shaderAst.$y(cs)), shaderAst.$x(cs)))
        ];
    });
    const rotationY3 = shaderAst.defn("mat3", "rotationY3", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat3(shaderAst.$x(cs), 0, shaderAst.neg(shaderAst.$y(cs)), 0, 1, 0, shaderAst.$y(cs), 0, shaderAst.$x(cs)))
        ];
    });
    const rotationZ3 = shaderAst.defn("mat3", "rotationZ3", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat3(shaderAst.$x(cs), shaderAst.$y(cs), 0, shaderAst.neg(shaderAst.$y(cs)), shaderAst.$x(cs), 0, 0, 0, 1))
        ];
    });
    const rotationX4 = shaderAst.defn("mat4", "rotationX4", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat4(1, 0, 0, 0, 0, shaderAst.$x(cs), shaderAst.$y(cs), 0, 0, shaderAst.neg(shaderAst.$y(cs)), shaderAst.$x(cs), 0, 0, 0, 0, 1))
        ];
    });
    const rotationY4 = shaderAst.defn("mat4", "rotationY4", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat4(shaderAst.$x(cs), 0, shaderAst.neg(shaderAst.$y(cs)), 0, 0, 1, 0, 0, shaderAst.$y(cs), 0, shaderAst.$x(cs), 0, 0, 0, 0, 1))
        ];
    });
    const rotationZ4 = shaderAst.defn("mat4", "rotationZ4", [["float"]], (theta) => {
        let cs;
        return [
            (cs = shaderAst.sym(cossin(theta))),
            shaderAst.ret(shaderAst.mat4(shaderAst.$x(cs), shaderAst.$y(cs), 0, 0, shaderAst.neg(shaderAst.$y(cs)), shaderAst.$x(cs), 0, 0, 0, 0, 1, 0, 0, 0, 0, 1))
        ];
    });

    const hash2 = shaderAst.defn("vec2", "hash2", [["vec2"]], (p) => [
        shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.sin(shaderAst.mul(p, shaderAst.mat2(127.1, 311.7, 269.5, 183.3))), 43758.5453)))
    ]);
    const hash3 = shaderAst.defn("vec3", "hash3", [["vec2"]], (p) => [
        shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.sin(shaderAst.vec3(shaderAst.dot(p, shaderAst.vec2(127.1, 311.7)), shaderAst.dot(p, shaderAst.vec2(269.5, 183.3)), shaderAst.dot(p, shaderAst.vec2(419.2, 371.9)))), 43758.5453)))
    ]);
    const H = shaderAst.vec3(0.1031, 0.103, 0.0973);
    const H4 = shaderAst.vec4(0.1031, 0.103, 0.0973, 0.1099);
    const hash11 = shaderAst.defn("float", "hash11", [["float"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, 0.1031)))),
            shaderAst.assign(x, shaderAst.mul(x, shaderAst.add(x, 19.19))),
            shaderAst.assign(x, shaderAst.mul(x, shaderAst.add(x, x))),
            shaderAst.ret(shaderAst.fract(x))
        ];
    });
    const hash12 = shaderAst.defn("float", "hash12", [["vec2"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyx"), 0.1031)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$x(x), shaderAst.$y(x)), shaderAst.$z(x))))
        ];
    });
    const hash13 = shaderAst.defn("float", "hash13", [["vec3"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyx"), 0.1031)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$x(x), shaderAst.$y(x)), shaderAst.$z(x))))
        ];
    });
    const hash21 = shaderAst.defn("vec2", "hash21", [["float"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.vec3(p), H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xx"), shaderAst.$(x, "yz")), shaderAst.$(x, "zy"))))
        ];
    });
    const hash22 = shaderAst.defn("vec2", "hash22", [["vec2"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyx"), H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xx"), shaderAst.$(x, "yz")), shaderAst.$(x, "zy"))))
        ];
    });
    const hash23 = shaderAst.defn("vec2", "hash23", [["vec3"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xx"), shaderAst.$(x, "yz")), shaderAst.$(x, "zy"))))
        ];
    });
    const hash31 = shaderAst.defn("vec3", "hash31", [["float"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxy"), shaderAst.$(x, "yzz")), shaderAst.$(x, "zyx"))))
        ];
    });
    const hash32 = shaderAst.defn("vec3", "hash32", [["vec2"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyx"), H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxy"), shaderAst.$(x, "yzz")), shaderAst.$(x, "zyx"))))
        ];
    });
    const hash33 = shaderAst.defn("vec3", "hash33", [["vec3"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, H)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "yzx"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxy"), shaderAst.$(x, "yzz")), shaderAst.$(x, "zyx"))))
        ];
    });
    const hash41 = shaderAst.defn("vec4", "hash41", [["float"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, H4)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "wzxy"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxyz"), shaderAst.$(x, "yzzw")), shaderAst.$(x, "zywx"))))
        ];
    });
    const hash42 = shaderAst.defn("vec4", "hash42", [["vec2"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyxy"), H4)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "wzxy"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxyz"), shaderAst.$(x, "yzzw")), shaderAst.$(x, "zywx"))))
        ];
    });
    const hash43 = shaderAst.defn("vec4", "hash43", [["vec3"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(shaderAst.$(p, "xyzx"), H4)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "wzxy"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxyz"), shaderAst.$(x, "yzzw")), shaderAst.$(x, "zywx"))))
        ];
    });
    const hash44 = shaderAst.defn("vec4", "hash44", [["vec4"]], (p) => {
        let x;
        return [
            (x = shaderAst.sym(shaderAst.fract(shaderAst.mul(p, H4)))),
            shaderAst.assign(x, shaderAst.add(x, shaderAst.dot(x, shaderAst.add(shaderAst.$(x, "wzxy"), 19.19)))),
            shaderAst.ret(shaderAst.fract(shaderAst.mul(shaderAst.add(shaderAst.$(x, "xxyz"), shaderAst.$(x, "yzzw")), shaderAst.$(x, "zywx"))))
        ];
    });

    const permute = shaderAst.defn("float", "permute", [["float"]], (v) => [
        shaderAst.ret(shaderAst.mod(shaderAst.mul(v, shaderAst.add(shaderAst.mul(v, shaderAst.float(34)), shaderAst.FLOAT1)), shaderAst.float(289)))
    ]);
    const permute2 = shaderAst.defn("vec2", "permute2", [["vec2"]], (v) => [
        shaderAst.ret(shaderAst.mod(shaderAst.mul(v, shaderAst.add(shaderAst.mul(v, shaderAst.float(34)), shaderAst.FLOAT1)), shaderAst.float(289)))
    ]);
    const permute3 = shaderAst.defn("vec3", "permute3", [["vec3"]], (v) => [
        shaderAst.ret(shaderAst.mod(shaderAst.mul(v, shaderAst.add(shaderAst.mul(v, shaderAst.float(34)), shaderAst.FLOAT1)), shaderAst.float(289)))
    ]);
    const permute4 = shaderAst.defn("vec4", "permute4", [["vec4"]], (v) => [
        shaderAst.ret(shaderAst.mod(shaderAst.mul(v, shaderAst.add(shaderAst.mul(v, shaderAst.float(34)), shaderAst.FLOAT1)), shaderAst.float(289)))
    ]);

    const snoise2 = shaderAst.defn("float", "snoise2", [["vec2"]], (v) => {
        let C;
        let i;
        let i1;
        let x0;
        let x12;
        let p;
        let m;
        let x;
        let h;
        let ox;
        let a0;
        let g;
        return [
            (C = shaderAst.sym(shaderAst.vec4(0.211324865405187, 0.366025403784439, -0.577350269189626, 0.024390243902439))),
            (i = shaderAst.sym(shaderAst.floor(shaderAst.add(v, shaderAst.dot(v, shaderAst.$(C, "yy")))))),
            (x0 = shaderAst.sym(shaderAst.add(shaderAst.sub(v, i), shaderAst.dot(i, shaderAst.$(C, "xx"))))),
            (i1 = shaderAst.sym(shaderAst.ternary(shaderAst.gt(shaderAst.$x(x0), shaderAst.$y(x0)), shaderAst.vec2(1, 0), shaderAst.vec2(0, 1)))),
            (x12 = shaderAst.sym(shaderAst.sub(shaderAst.add(shaderAst.$(x0, "xyxy"), shaderAst.$(C, "xxzz")), shaderAst.vec4(i1, 0, 0)))),
            shaderAst.assign(i, shaderAst.mod(i, shaderAst.float(289))),
            (p = shaderAst.sym(permute3(shaderAst.add(permute3(shaderAst.add(shaderAst.vec3(0, shaderAst.$y(i1), 1), shaderAst.$y(i))), shaderAst.add(shaderAst.vec3(0, shaderAst.$x(i1), 1), shaderAst.$x(i)))))),
            (m = shaderAst.sym(shaderAst.max(shaderAst.sub(shaderAst.FLOAT05, shaderAst.vec3(magSq2(x0), magSq2(shaderAst.$xy(x12)), magSq2(shaderAst.$(x12, "zw")))), shaderAst.vec3()))),
            shaderAst.assign(m, shaderAst.mul(m, m)),
            shaderAst.assign(m, shaderAst.mul(m, m)),
            (x = shaderAst.sym(shaderAst.sub(shaderAst.mul(2, shaderAst.fract(shaderAst.mul(p, shaderAst.$(C, "www")))), shaderAst.FLOAT1))),
            (h = shaderAst.sym(shaderAst.sub(shaderAst.abs(x), shaderAst.FLOAT05))),
            (ox = shaderAst.sym(shaderAst.floor(shaderAst.add(x, shaderAst.FLOAT05)))),
            (a0 = shaderAst.sym(shaderAst.sub(x, ox))),
            shaderAst.assign(m, shaderAst.mul(m, shaderAst.sub(1.79284291400159, shaderAst.mul(0.85373472095314, shaderAst.add(shaderAst.mul(a0, a0), shaderAst.mul(h, h)))))),
            (g = shaderAst.sym(shaderAst.vec3(shaderAst.add(shaderAst.mul(shaderAst.$x(a0), shaderAst.$x(x0)), shaderAst.mul(shaderAst.$x(h), shaderAst.$y(x0)))))),
            shaderAst.assign(shaderAst.$(g, "yz"), shaderAst.add(shaderAst.mul(shaderAst.$(a0, "yz"), shaderAst.$(x12, "xz")), shaderAst.mul(shaderAst.$(h, "yz"), shaderAst.$(x12, "yw")))),
            shaderAst.ret(shaderAst.mul(130, shaderAst.dot(m, g)))
        ];
    });

    const voronoise2 = shaderAst.defn("float", "voronoise2", [["vec2"], ["float"], ["float"]], (x, u, v) => {
        let p;
        let f;
        let coeff;
        let k;
        let va;
        let wt;
        let g;
        let o;
        let r;
        let w;
        return [
            (p = shaderAst.sym(shaderAst.floor(x))),
            (f = shaderAst.sym(shaderAst.fract(x))),
            (coeff = shaderAst.sym(shaderAst.vec3(u, u, 1))),
            (k = shaderAst.sym(shaderAst.add(1, shaderAst.mul(63, shaderAst.pow(shaderAst.sub(1, v), shaderAst.float(4)))))),
            (va = shaderAst.sym(shaderAst.FLOAT0)),
            (wt = shaderAst.sym(shaderAst.FLOAT0)),
            shaderAst.forLoop(shaderAst.sym(shaderAst.int(-2)), (i) => shaderAst.lte(i, shaderAst.int(2)), shaderAst.inc, (i) => [
                shaderAst.forLoop(shaderAst.sym(shaderAst.int(-2)), (j) => shaderAst.lte(j, shaderAst.int(2)), shaderAst.inc, (j) => [
                    (g = shaderAst.sym(shaderAst.vec2(shaderAst.float(i), shaderAst.float(j)))),
                    (o = shaderAst.sym(shaderAst.mul(hash32(shaderAst.add(p, g)), coeff))),
                    (r = shaderAst.sym(shaderAst.add(shaderAst.sub(g, f), shaderAst.$xy(o)))),
                    (w = shaderAst.sym(shaderAst.pow(shaderAst.sub(1, shaderAst.smoothstep(shaderAst.FLOAT0, shaderAst.SQRT2, shaderAst.sqrt(shaderAst.dot(r, r)))), k))),
                    shaderAst.assign(va, shaderAst.madd(w, shaderAst.$z(o), va)),
                    shaderAst.assign(wt, shaderAst.add(wt, w))
                ])
            ]),
            shaderAst.ret(shaderAst.div(va, wt))
        ];
    });

    const worleyDist = shaderAst.defn("vec3", "worleyDist", [["vec3"], ["vec3"]], (a, b) => [shaderAst.ret(shaderAst.add(shaderAst.mul(a, a), shaderAst.mul(b, b)))]);
    const worleyDistManhattan = shaderAst.defn("vec3", "worleyDistManhatten", [["vec3"], ["vec3"]], (a, b) => [shaderAst.ret(shaderAst.add(shaderAst.abs(a), shaderAst.abs(b)))]);
    const worley2 = (distFn) => shaderAst.defn("vec2", "worley2", [["vec2"], ["float"]], (P, jitter) => {
        const K = shaderAst.float(1 / 7);
        const Ko = shaderAst.float(3 / 7);
        const oI = shaderAst.sym(shaderAst.vec3(-1, 0, 1));
        const oF = shaderAst.sym(shaderAst.vec3(-0.5, 0.5, 1.5));
        let pI;
        let pF;
        let px;
        let p;
        let d1;
        let d2;
        let d3;
        let d1a;
        const $dist = (x) => distFn(shaderAst.add(shaderAst.add(shaderAst.$x(pF), x), shaderAst.mul(shaderAst.sub(shaderAst.fract(shaderAst.mul(p, K)), Ko), jitter)), shaderAst.add(shaderAst.sub(shaderAst.$y(pF), oF), shaderAst.mul(shaderAst.sub(shaderAst.mul(shaderAst.mod(shaderAst.floor(shaderAst.mul(p, K)), shaderAst.float(7)), K), Ko), jitter)));
        return [
            oI,
            oF,
            (pI = shaderAst.sym(shaderAst.mod(shaderAst.floor(P), shaderAst.float(289)))),
            (pF = shaderAst.sym(shaderAst.fract(P))),
            (px = shaderAst.sym(permute3(shaderAst.add(oI, shaderAst.$x(pI))))),
            (p = shaderAst.sym(permute3(shaderAst.add(shaderAst.add(oI, shaderAst.$y(pI)), shaderAst.$x(px))))),
            (d1 = shaderAst.sym($dist(0.5))),
            shaderAst.assign(p, permute3(shaderAst.add(oI, shaderAst.add(shaderAst.$y(pI), shaderAst.$y(px))))),
            (d2 = shaderAst.sym($dist(-0.5))),
            shaderAst.assign(p, permute3(shaderAst.add(oI, shaderAst.add(shaderAst.$y(pI), shaderAst.$z(px))))),
            (d3 = shaderAst.sym($dist(-1.5))),
            (d1a = shaderAst.sym(shaderAst.min(d1, d2))),
            shaderAst.assign(d2, shaderAst.min(shaderAst.max(d1, d2), d3)),
            shaderAst.assign(d1, shaderAst.min(d1a, d2)),
            shaderAst.assign(d2, shaderAst.max(d1a, d2)),
            shaderAst.assign(shaderAst.$xy(d1), shaderAst.ternary(shaderAst.lt(shaderAst.$x(d1), shaderAst.$y(d1)), shaderAst.$xy(d1), shaderAst.$(d1, "yx"))),
            shaderAst.assign(shaderAst.$(d1, "xz"), shaderAst.ternary(shaderAst.lt(shaderAst.$x(d1), shaderAst.$z(d1)), shaderAst.$(d1, "xz"), shaderAst.$(d1, "zx"))),
            shaderAst.assign(shaderAst.$(d1, "yz"), shaderAst.min(shaderAst.$(d1, "yz"), shaderAst.$(d2, "yz"))),
            shaderAst.assign(shaderAst.$y(d1), shaderAst.min(shaderAst.min(shaderAst.$y(d1), shaderAst.$z(d1)), shaderAst.$x(d2))),
            shaderAst.ret(shaderAst.sqrt(shaderAst.$xy(d1)))
        ];
    });

    const raymarchAO = (scene, numSamples = 5) => shaderAst.defn("float", "raymarchAO", [["vec3"], ["vec3"]], (p, n) => {
        let r;
        let w;
        let d0;
        return [
            (r = shaderAst.sym(shaderAst.FLOAT0)),
            (w = shaderAst.sym(shaderAst.FLOAT1)),
            shaderAst.forLoop(shaderAst.sym("float", shaderAst.float(1)), (i) => shaderAst.lte(i, shaderAst.float(numSamples)), (i) => shaderAst.inc(i), (i) => [
                (d0 = shaderAst.sym(shaderAst.mul(i, 1 / numSamples))),
                shaderAst.assign(r, shaderAst.add(r, shaderAst.mul(w, shaderAst.sub(d0, shaderAst.$x(scene(shaderAst.add(p, shaderAst.mul(n, d0)))))))),
                shaderAst.assign(w, shaderAst.mul(w, shaderAst.FLOAT05))
            ]),
            shaderAst.ret(shaderAst.sub(shaderAst.FLOAT1, clamp01(r)))
        ];
    });

    const raymarchDir = shaderAst.defn("vec3", "raymarchDir", [["vec2"], ["vec2"], ["float"]], (frag, res, fov) => {
        let uv;
        return [
            (uv = shaderAst.sym(shaderAst.sub(frag, shaderAst.div(res, shaderAst.FLOAT2)))),
            shaderAst.ret(shaderAst.normalize(shaderAst.vec3(uv, shaderAst.neg(shaderAst.div(shaderAst.$y(res), shaderAst.tan(shaderAst.div(shaderAst.radians(fov), shaderAst.FLOAT2)))))))
        ];
    });

    const raymarchNormal = (scene, name = "raymarchNormal") => shaderAst.defn("vec3", name, [["vec3"], ["float"]], (p, smooth) => {
        let dn;
        const comp = (id) => shaderAst.sub(shaderAst.$x(scene(shaderAst.add(p, shaderAst.$(dn, id)))), shaderAst.$x(scene(shaderAst.sub(p, shaderAst.$(dn, id)))));
        return [
            (dn = shaderAst.sym(shaderAst.vec2(smooth, 0))),
            shaderAst.ret(shaderAst.normalize(shaderAst.vec3(comp("xyy"), comp("yxy"), comp("yyx"))))
        ];
    });

    const rayPointAt = (p, dir, t, norm = false) => shaderAst.add(p, shaderAst.mul(norm ? shaderAst.normalize(dir) : dir, t));

    const raymarchScene = (scene, _opts) => {
        const opts = Object.assign({ name: "raymarchScene", near: 0.1, far: 10, steps: 100, eps: 0.01, bias: 0.7 }, _opts);
        return shaderAst.defn("vec2", opts.name, [["vec3"], ["vec3"]], (pos, dir) => {
            let total;
            let res;
            return [
                (total = shaderAst.sym(shaderAst.float(opts.near))),
                (res = shaderAst.sym("vec2")),
                shaderAst.forLoop(shaderAst.sym("int", shaderAst.int(0)), (i) => shaderAst.lt(i, shaderAst.int(opts.steps)), (i) => shaderAst.inc(i), () => [
                    shaderAst.assign(res, scene(rayPointAt(pos, dir, total))),
                    shaderAst.ifThen(shaderAst.lt(shaderAst.$x(res), shaderAst.float(opts.eps)), [
                        shaderAst.ret(shaderAst.vec2(total, shaderAst.$y(res)))
                    ]),
                    shaderAst.assign(total, shaderAst.madd(shaderAst.$x(res), shaderAst.float(opts.bias), total)),
                    shaderAst.ifThen(shaderAst.gt(total, shaderAst.float(opts.far)), [shaderAst.brk])
                ]),
                shaderAst.ret(shaderAst.vec2(opts.far, 0))
            ];
        });
    };

    const aspectCorrectedUV = shaderAst.defn("vec2", "aspectCorrectedUV", [["vec2"], ["vec2"]], (pos, res) => {
        let uv;
        return [
            (uv = shaderAst.sym("vec2", fit0111(shaderAst.div(pos, res)))),
            shaderAst.assign(shaderAst.$x(uv), shaderAst.mul(shaderAst.$x(uv), shaderAst.div(shaderAst.$x(res), shaderAst.$y(res)))),
            shaderAst.ret(uv)
        ];
    });

    const sdfAnnular = (d, r) => shaderAst.sub(shaderAst.abs(d), r);

    const sdfBox2 = shaderAst.defn("float", "sdRect", [["vec2"], ["vec2"]], (p, size) => {
        let d;
        return [
            (d = shaderAst.sym(shaderAst.sub(shaderAst.abs(p), size))),
            shaderAst.ret(shaderAst.add(shaderAst.min(maxComp2(d), shaderAst.FLOAT0), shaderAst.length(shaderAst.max(d, shaderAst.vec2()))))
        ];
    });
    const sdfBox3 = shaderAst.defn("float", "sdAABB", [["vec3"], ["vec3"]], (p, size) => {
        let d;
        return [
            (d = shaderAst.sym(shaderAst.sub(shaderAst.abs(p), size))),
            shaderAst.ret(shaderAst.add(shaderAst.min(maxComp3(d), shaderAst.FLOAT0), shaderAst.length(shaderAst.max(d, shaderAst.vec3()))))
        ];
    });

    const sdfCylinder = shaderAst.defn("float", "sdCylinder", [["vec3"], ["float"], ["float"]], (p, h, r) => {
        let d;
        return [
            (d = shaderAst.sym(shaderAst.sub(shaderAst.abs(shaderAst.vec2(shaderAst.length(shaderAst.$(p, "xz")), shaderAst.$y(p))), shaderAst.vec2(h, r)))),
            shaderAst.ret(shaderAst.add(shaderAst.min(maxComp2(d), shaderAst.FLOAT0), shaderAst.length(shaderAst.max(d, shaderAst.vec2()))))
        ];
    });

    const sdfIntersect = (a, b) => shaderAst.max(b, a);

    const line = (p, a, b) => {
        let pa, ba;
        return [
            (pa = shaderAst.sym(shaderAst.sub(p, a))),
            (ba = shaderAst.sym(shaderAst.sub(b, a))),
            shaderAst.ret(shaderAst.length(shaderAst.sub(pa, shaderAst.mul(ba, clamp01(shaderAst.div(shaderAst.dot(pa, ba), shaderAst.dot(ba, ba)))))))
        ];
    };
    const sdfLine2 = shaderAst.defn("float", "sdLine2", [["vec2"], ["vec2"], ["vec2"]], line);
    const sdfLine3 = shaderAst.defn("float", "sdLine3", [["vec3"], ["vec3"], ["vec3"]], line);

    const sdfPlane2 = shaderAst.defn("float", "sdPlane", [["vec2"], ["vec2"], ["float"]], (p, n, w) => [shaderAst.ret(shaderAst.add(shaderAst.dot(p, n), w))]);
    const sdfPlane3 = shaderAst.defn("float", "sdPlane", [["vec3"], ["vec3"], ["float"]], (p, n, w) => [shaderAst.ret(shaderAst.add(shaderAst.dot(p, n), w))]);

    const sdfRepeat2 = shaderAst.defn("vec2", "sdTxRepeat3", [["vec2"], ["vec2"]], (p, c) => [shaderAst.ret(shaderAst.sub(shaderAst.mod(p, c), shaderAst.mul(c, shaderAst.FLOAT05)))]);
    const sdfRepeat3 = shaderAst.defn("vec3", "sdTxRepeat3", [["vec3"], ["vec3"]], (p, c) => [shaderAst.ret(shaderAst.sub(shaderAst.mod(p, c), shaderAst.mul(c, shaderAst.FLOAT05)))]);

    const sdfRound = (d, r) => shaderAst.sub(d, r);

    const sdfSmoothIntersect = shaderAst.defn("float", "sdOpSmoothIntersect", [["float"], ["float"], ["float"]], (a, b, k) => {
        let h;
        return [
            (h = shaderAst.sym(clamp01(fit1101(shaderAst.div(shaderAst.sub(b, a), k))))),
            shaderAst.ret(shaderAst.add(shaderAst.mix(b, a, h), shaderAst.mul(shaderAst.mul(k, h), shaderAst.sub(shaderAst.FLOAT1, h))))
        ];
    });

    const sdfSmoothSubtract = shaderAst.defn("float", "sdOpSmoothSubtract", [["float"], ["float"], ["float"]], (a, b, k) => {
        let h;
        return [
            (h = shaderAst.sym(clamp01(fit1101(shaderAst.div(shaderAst.add(b, a), k))))),
            shaderAst.ret(shaderAst.add(shaderAst.mix(b, shaderAst.neg(a), h), shaderAst.mul(shaderAst.mul(k, h), shaderAst.sub(shaderAst.FLOAT1, h))))
        ];
    });

    const sdfSmoothUnion = shaderAst.defn("float", "sdOpSmoothUnion", [["float"], ["float"], ["float"]], (a, b, k) => {
        let h;
        return [
            (h = shaderAst.sym(clamp01(fit1101(shaderAst.div(shaderAst.sub(b, a), k))))),
            shaderAst.ret(shaderAst.sub(shaderAst.mix(b, a, h), shaderAst.mul(shaderAst.mul(k, h), shaderAst.sub(shaderAst.FLOAT1, h))))
        ];
    });

    const sdfCircle = shaderAst.defn("float", "sdCircle", [["vec2"], ["float"]], (p, r) => [shaderAst.ret(shaderAst.sub(shaderAst.length(p), r))]);
    const sdfSphere = shaderAst.defn("float", "sdSphere", [["vec3"], ["float"]], (p, r) => [shaderAst.ret(shaderAst.sub(shaderAst.length(p), r))]);

    const sdfTorus = shaderAst.defn("float", "sdTorus", [["vec3"], ["float"], ["float"]], (p, r1, r2) => [
        shaderAst.ret(shaderAst.sub(shaderAst.length(shaderAst.vec2(shaderAst.sub(shaderAst.length(shaderAst.$(p, "xz")), r2), shaderAst.$y(p))), r1))
    ]);

    const sdfTriangle2 = shaderAst.defn("float", "sdTriangle", [["vec2"], ["vec2"], ["vec2"], ["vec2"]], (p, a, b, c) => {
        let e0, e1, e2;
        let v0, v1, v2;
        let pq0, pq1, pq2;
        let s;
        let d;
        const $pq = (v, e) => shaderAst.sub(v, shaderAst.mul(e, clamp01(shaderAst.div(shaderAst.dot(v, e), shaderAst.dot(e, e)))));
        return [
            (e0 = shaderAst.sym(shaderAst.sub(b, a))),
            (e1 = shaderAst.sym(shaderAst.sub(c, b))),
            (e2 = shaderAst.sym(shaderAst.sub(a, c))),
            (v0 = shaderAst.sym(shaderAst.sub(p, a))),
            (v1 = shaderAst.sym(shaderAst.sub(p, b))),
            (v2 = shaderAst.sym(shaderAst.sub(p, c))),
            (pq0 = shaderAst.sym($pq(v0, e0))),
            (pq1 = shaderAst.sym($pq(v1, e1))),
            (pq2 = shaderAst.sym($pq(v2, e2))),
            (s = shaderAst.sym(shaderAst.sign(cross2(e0, e2)))),
            (d = shaderAst.sym(shaderAst.min(shaderAst.min(shaderAst.vec2(shaderAst.dot(pq0, pq0), shaderAst.mul(s, cross2(v0, e0))), shaderAst.vec2(shaderAst.dot(pq1, pq1), shaderAst.mul(s, cross2(v1, e1)))), shaderAst.vec2(shaderAst.dot(pq2, pq2), shaderAst.mul(s, cross2(v2, e2)))))),
            shaderAst.ret(shaderAst.mul(shaderAst.neg(shaderAst.sqrt(shaderAst.$x(d))), shaderAst.sign(shaderAst.$y(d))))
        ];
    });

    const sdfUnion = (a, b) => shaderAst.min(a, b);

    const indexToCoord = (i, width) => shaderAst.uvec2(shaderAst.modi(i, width), shaderAst.div(i, width));
    const coordToIndex = (coord, width) => shaderAst.madd(shaderAst.$y(coord), width, shaderAst.$x(coord));

    const indexToUV = shaderAst.defn("vec2", "indexToUV", [["int", "i", { prec: "highp" }], ["ivec2"]], (i, size) => [
        shaderAst.ret(shaderAst.vec2(shaderAst.div(shaderAst.float(shaderAst.modi(i, shaderAst.$x(size))), shaderAst.float(shaderAst.$x(size))), shaderAst.div(shaderAst.float(shaderAst.div(i, shaderAst.$x(size))), shaderAst.float(shaderAst.$y(size)))))
    ]);
    const uvToIndex = shaderAst.defn("int", "uvToIndex", [["vec2"], ["int", "width", { prec: "highp" }]], (uv, width) => [
        shaderAst.ret(shaderAst.add(shaderAst.int(shaderAst.mul(shaderAst.$x(uv), shaderAst.float(width))), shaderAst.int(shaderAst.mul(shaderAst.$y(uv), shaderAst.float(shaderAst.mul(width, width))))))
    ]);

    const readIndex1 = (tex, i, size) => shaderAst.$x(shaderAst.texture(tex, indexToUV(i, size)));
    const readIndex2 = (tex, i, size) => shaderAst.$xy(shaderAst.texture(tex, indexToUV(i, size)));
    const readIndex3 = (tex, i, size) => shaderAst.$xyz(shaderAst.texture(tex, indexToUV(i, size)));
    const readIndex4 = (tex, i, size) => shaderAst.texture(tex, indexToUV(i, size));

    exports.additive = additive;
    exports.aspectCorrectedUV = aspectCorrectedUV;
    exports.cartesian2 = cartesian2;
    exports.cartesian3 = cartesian3;
    exports.clamp01 = clamp01;
    exports.clamp11 = clamp11;
    exports.coordToIndex = coordToIndex;
    exports.cossin = cossin;
    exports.cross2 = cross2;
    exports.crossC2 = crossC2;
    exports.diffuseLighting = diffuseLighting;
    exports.distChebyshev2 = distChebyshev2;
    exports.distChebyshev3 = distChebyshev3;
    exports.distChebyshev4 = distChebyshev4;
    exports.distManhattan2 = distManhattan2;
    exports.distManhattan3 = distManhattan3;
    exports.distManhattan4 = distManhattan4;
    exports.fit01 = fit01;
    exports.fit0111 = fit0111;
    exports.fit1101 = fit1101;
    exports.fogExp = fogExp;
    exports.fogExp2 = fogExp2;
    exports.fogLinear = fogLinear;
    exports.halfLambert = halfLambert;
    exports.hash11 = hash11;
    exports.hash12 = hash12;
    exports.hash13 = hash13;
    exports.hash2 = hash2;
    exports.hash21 = hash21;
    exports.hash22 = hash22;
    exports.hash23 = hash23;
    exports.hash3 = hash3;
    exports.hash31 = hash31;
    exports.hash32 = hash32;
    exports.hash33 = hash33;
    exports.hash41 = hash41;
    exports.hash42 = hash42;
    exports.hash43 = hash43;
    exports.hash44 = hash44;
    exports.indexToCoord = indexToCoord;
    exports.indexToUV = indexToUV;
    exports.lambert = lambert;
    exports.lookat = lookat;
    exports.luminanceRGB = luminanceRGB;
    exports.magSq2 = magSq2;
    exports.magSq3 = magSq3;
    exports.magSq4 = magSq4;
    exports.maxComp2 = maxComp2;
    exports.maxComp3 = maxComp3;
    exports.maxComp4 = maxComp4;
    exports.minComp2 = minComp2;
    exports.minComp3 = minComp3;
    exports.minComp4 = minComp4;
    exports.orthogonal3 = orthogonal3;
    exports.permute = permute;
    exports.permute2 = permute2;
    exports.permute3 = permute3;
    exports.permute4 = permute4;
    exports.perpendicularCCW = perpendicularCCW;
    exports.perpendicularCW = perpendicularCW;
    exports.polar2 = polar2;
    exports.polar3 = polar3;
    exports.rayPointAt = rayPointAt;
    exports.raymarchAO = raymarchAO;
    exports.raymarchDir = raymarchDir;
    exports.raymarchNormal = raymarchNormal;
    exports.raymarchScene = raymarchScene;
    exports.readIndex1 = readIndex1;
    exports.readIndex2 = readIndex2;
    exports.readIndex3 = readIndex3;
    exports.readIndex4 = readIndex4;
    exports.rotation2 = rotation2;
    exports.rotationX3 = rotationX3;
    exports.rotationX4 = rotationX4;
    exports.rotationY3 = rotationY3;
    exports.rotationY4 = rotationY4;
    exports.rotationZ3 = rotationZ3;
    exports.rotationZ4 = rotationZ4;
    exports.sdfAnnular = sdfAnnular;
    exports.sdfBox2 = sdfBox2;
    exports.sdfBox3 = sdfBox3;
    exports.sdfCircle = sdfCircle;
    exports.sdfCylinder = sdfCylinder;
    exports.sdfIntersect = sdfIntersect;
    exports.sdfLine2 = sdfLine2;
    exports.sdfLine3 = sdfLine3;
    exports.sdfPlane2 = sdfPlane2;
    exports.sdfPlane3 = sdfPlane3;
    exports.sdfRepeat2 = sdfRepeat2;
    exports.sdfRepeat3 = sdfRepeat3;
    exports.sdfRound = sdfRound;
    exports.sdfSmoothIntersect = sdfSmoothIntersect;
    exports.sdfSmoothSubtract = sdfSmoothSubtract;
    exports.sdfSmoothUnion = sdfSmoothUnion;
    exports.sdfSphere = sdfSphere;
    exports.sdfTorus = sdfTorus;
    exports.sdfTriangle2 = sdfTriangle2;
    exports.sdfUnion = sdfUnion;
    exports.sincos = sincos;
    exports.snoise2 = snoise2;
    exports.surfaceNormal = surfaceNormal;
    exports.toLinear = toLinear;
    exports.toSRGB = toSRGB;
    exports.transformMVP = transformMVP;
    exports.trilight = trilight;
    exports.uvToIndex = uvToIndex;
    exports.voronoise2 = voronoise2;
    exports.worley2 = worley2;
    exports.worleyDist = worleyDist;
    exports.worleyDistManhattan = worleyDistManhattan;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
