import { $x, add, assign, defn, float, FLOAT0, FLOAT05, FLOAT1, forLoop, inc, lte, mul, ret, sub, sym } from "@thi.ng/shader-ast";
import { clamp01 } from "../math/clamp";
/**
 * Higher order function returning an function to compute the Ambient
 * Occlusion term / shadow factor for given SDF scene function. The
 * returned function takes 2 arguments: surface pos and normal. It
 * returns a float in [0..1] interval (zero = fully occluded).
 *
 * @param scene
 * @param numSamples
 */
export const raymarchAO = (scene, numSamples = 5) => defn("float", "raymarchAO", [["vec3"], ["vec3"]], (p, n) => {
    let r;
    let w;
    let d0;
    return [
        (r = sym(FLOAT0)),
        (w = sym(FLOAT1)),
        forLoop(sym("float", float(1)), (i) => lte(i, float(numSamples)), (i) => inc(i), (i) => [
            (d0 = sym(mul(i, 1 / numSamples))),
            assign(r, add(r, mul(w, sub(d0, $x(scene(add(p, mul(n, d0)))))))),
            assign(w, mul(w, FLOAT05))
        ]),
        ret(sub(FLOAT1, clamp01(r)))
    ];
});
